package net.java.dev.fwutil.demoapp;

import net.java.dev.fwutil.Labels;
import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.LogUI;
import net.java.dev.fwutil.Restartable;
import net.java.dev.fwutil.Settings;
import net.java.dev.fwutil.Startup;
import net.java.dev.fwutil.UIMessages;

public class DemoApp implements Restartable{

	// Needed in case of restart - MainFrame passes this Restartable on to
	// StartUp.close(Restartable);
	private static final DemoApp thisInstance = new DemoApp();
	
	private DemoApp() {
		super();
	}
	
	public static DemoApp getInstance() {
		return thisInstance;
	}

	/** The restart method as defined by the Restartable interface. 
	 * Simply calls main again.
	 */
	public void restart(String[] args) {
		main(args);
	}

	public static void main(String[] args) {
		
		Settings.appName = "FWUtil demo application";
		Settings.appVersion = "0.1";
		Settings.defaultSettingsFile = "DemoAppSettings.xml";
		Labels.labelFile = "resources/demolabels.txt";
		if (!Startup.boot(args, Settings.appName, Settings.appVersion, null)) {
			Log.log("Startup failed, exiting.");
			UIMessages.showErrorMessage(LogUI.getInstance(), "StartUp error", "Error during startup, please examine log.\nClosing " + Settings.appName);
			Log.close();
			System.exit(1);
		}
		if (Settings.firstStart && Settings.get(Settings.defaultFontSize) == null) { // No font(size) previous selected
			Settings.put(Settings.defaultFontSize, 12);
			Log.log("First start, default font size set to 12");
			Settings.setDefaultFont();
		}
		Log.log(Labels.get("MainFrameStart"));
		//LogUI.setProgrammaticClose();
		DMainWindow.openWindow(DMainWindow.class, thisInstance, null);
	}

}
