package net.java.dev.fwutil;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

/** A (hash)map that stores multiple value for one key in a list. */
public class ListMap<K, V> extends HashMap<K, List<V>> {

	private static final long serialVersionUID = 5492077456821986863L;
	
	/** Set to true to allow only unique values to be stored for a key. */
	public boolean uniqueValues = false;

	public ListMap() {
		super();
	}
	
	/** Returns true when value is stored for key in the map. */
	public boolean containsValue(final K key, final V value) {

		if (!containsKey(key)) return false;
		List<V> l = get(key);
		return (l.contains(value));
	}

	/** Returns true when value is stored in the map. */
	public boolean containsAValue(final V value) {

		Iterator<K> it = keySet().iterator();
		while (it.hasNext()) {
			K key = it.next();
			List<V> l = get(key);
			if (l != null && l.contains(value)) return true;
		}
		return false;
	}
	
	/** Sets this one value as the only value for the key (stored in a list). */
	public void set(final K key, final V value) {

		List<V> l = get(key);
		if (l != null) {
			l.clear();
			l.add(value);
		} else {
			l = new ArrayList<V>();
			l.add(value);
			put(key, l);
		}
	}
	
	/** Returns a clone of the given list. */
	public List<V> clone(final List<V> original) {
		
		if (original == null) return null;
		List<V> clone = new ArrayList<V>();
		for (int i = 0; i < original.size(); i++) clone.add(original.get(i));
		return clone;
	}

	/** Adds the value to the list of values for key.
	 * Uses boolean uniqueValues. */
	public void add(final K key, final V value) {

		List<V> l = get(key);
		if (l == null) {
			l = new ArrayList<V>();
			put(key, l);
			l.add(value);
		} else {
			if (uniqueValues) { 
				if (!l.contains(value)) l.add(value);
			} else {
				l.add(value);
			}
		}
	}

	/** Removes the value from the list of values for key. */
	public boolean remove(final K key, final V value) {

		List<V> l = get(key);
		if (l == null) return false;
		boolean removed = l.remove(value);
		if (l.size() == 0) remove(key);
		return removed;
	}
	
	/** Clears the lists and then calls super.clear(). */
	public void clear() {
		
		final Iterator<K> it = keySet().iterator();
		while (it.hasNext()) {
			K key = it.next();
			if (get(key) != null) get(key).clear();
		}
		super.clear();
	}
}
