package net.java.dev.fwutil;

import java.awt.Color;
import java.awt.KeyboardFocusManager;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.JViewport;
import javax.swing.KeyStroke;
import javax.swing.SortOrder;
import javax.swing.RowSorter.SortKey;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;

import net.java.dev.fwutil.Settings;
import net.java.dev.hvlayout.HVProps;
import net.java.dev.hvlayout.HVTableHeader;

/** A Table that remembers column sizes by column name. **/
public class MTable extends JTable {
	
	private static final long serialVersionUID = -1048241619253266530L;
	
	protected String tableId;
	
	public MTable(String tableIdentiefier) {
		super();
		tableId = tableIdentiefier;
		setTableHeader(new HVTableHeader(getColumnModel()));
		setAutoResizeMode(AUTO_RESIZE_OFF);
		setRowMargin(2);
		setRowHeight(HVProps.defaultLineHeight - HVProps.smallHeightCorrection + 2);
		setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
		setColumnSelectionAllowed(false);
		setCellSelectionEnabled(false);
		setRowSelectionAllowed(true);
		setBackground(Color.WHITE);
		setForeground(Color.BLACK);
		setSelectionBackground(Color.LIGHT_GRAY);
		setSelectionForeground(Color.BLACK);
		setFocusable(true);
		setDefaultRenderer(Number.class, new TableCellNumberRenderer());
		setDefaultRenderer(Double.class, new TableCellNumberRenderer());
		setDefaultRenderer(Date.class, new TableCellDateRenderer());
		addTabKeyActions();
	}
	
	/**
	 * Show tooltip if text is truncated.
	 * Copied from http://www.codefund.com/515/setting-tooltip-jtable-5157346.shtm
	 */
	public String getToolTipText(final MouseEvent evt){

		Point point = evt.getPoint();
		int row = rowAtPoint(point), column = columnAtPoint(point);
		// Set cell text as tooltip if it doesn't completely fit in the cell
		if(row > -1 && column > -1 && getValueAt(row, column) != null) {
			String s = getValueAt(row, column).toString();
			if (getFontMetrics(getFont()).stringWidth(s) >
				getCellRect(row, column, false).width) {
				return s;
			}
		}
		return null;
	}

	/**
	 * Overloaded to make sure the table scrolls to the selected row.
	 * Copied from http://forum.java.sun.com/thread.jspa?threadID=332985&messageID=2808998
	 */
	public void setRowSelectionInterval(final int from, final int to) {
		super.setRowSelectionInterval(from, to);
		final Rectangle rect = getCellRect(from, 0, true);
		scrollRectToVisible(rect);
	}
	
	/**
	 * Scrolls one page down. Sets the selected row to the top of the list
	 * when currently selected row is out of view.
	 * @param viewport
	 */
	public void scrollPageDown(JViewport viewport) {
		
		final Rectangle viewRect = viewport.getViewRect();
		int firstRow = rowAtPoint(new Point(0, viewRect.y));
		int lastRow = rowAtPoint(new Point(0, viewRect.y + viewRect.height - 1));
		int rowsInView = lastRow - firstRow;
		if (rowsInView > 0 && lastRow < getRowCount()) {
			int rowSelected = getSelectedRow();
			if (rowSelected == lastRow) {
				firstRow = lastRow;
				lastRow = Math.min(getRowCount()-1, lastRow + rowsInView);
			} 
			setRowSelectionInterval(lastRow, lastRow);
			// Required to update icons in the table.
			if (getModel() instanceof AbstractTableModel) {
				((AbstractTableModel)getModel()).fireTableRowsUpdated(firstRow, lastRow);
			}
		} else {
			scrollToBottom();
		}
	}
	
	public void scrollToBottom() {
		if (getRowCount() > 0)
			setRowSelectionInterval(getRowCount()-1, getRowCount()-1);
	}


	/**
	 * Scrolls one page up. Sets the selected row to the bottom of the list
	 * when currently selected row is out of view.
	 * @param viewport
	 */
	public void scrollPageUp(JViewport viewport) {
		
		final Rectangle viewRect = viewport.getViewRect();
		int firstRow = rowAtPoint(new Point(0, viewRect.y));
		int lastRow = rowAtPoint(new Point(0, viewRect.y + viewRect.height - 1));
		int rowsInView = lastRow - firstRow;
		if (rowsInView > 0 && firstRow > -1) {
			int rowSelected = getSelectedRow();
			if (rowSelected == firstRow) {
				lastRow = firstRow;
				firstRow = Math.max(0, firstRow - rowsInView);
			}
			setRowSelectionInterval(firstRow, firstRow);
			// Required to update icons in the table.
			if (getModel() instanceof AbstractTableModel) {
				((AbstractTableModel)getModel()).fireTableRowsUpdated(firstRow, lastRow);
			}
		} else {
			scrollToTop();
		}
	}
	
	public void scrollToTop() {
		if (getRowCount() > -1)
			setRowSelectionInterval(0, 0);
	}
	
	/**
	 * Adds the action-keys for this table (tab and shift-tab).
	 */
	@SuppressWarnings("serial")
	private void addTabKeyActions() {
		/* Note that the two string literals in the put methods have to match. */

		// Process tab-key (normally jumps field in table)
		Action tabOut = new AbstractAction(){
			public void actionPerformed(final ActionEvent ae){
				KeyboardFocusManager.getCurrentKeyboardFocusManager().focusNextComponent();
			}
		};
		getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0), "tabOut");
		getActionMap().put("tabOut", tabOut);

		// Process shift-tab key
		Action tabOutBack = new AbstractAction(){
			public void actionPerformed(final ActionEvent ae){
				KeyboardFocusManager.getCurrentKeyboardFocusManager().focusPreviousComponent();
			}
		};
		getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, InputEvent.SHIFT_MASK), "tabOutBack");
		getActionMap().put("tabOutBack", tabOutBack);
	}

	@SuppressWarnings("unchecked")
	protected HashMap<String, Object> getTableUIProps() {
		
		HashMap<String, Object> tableProps = (HashMap<String, Object>)Settings.get("TableUIProps");
		return tableProps;
	}
	
	protected void setTableUIProps(HashMap<String, Object> tableUIProps) {
		Settings.put("TableUIProps", tableUIProps);
	}

	@SuppressWarnings("unchecked")
	protected void storeUIProps() {

		HashMap<String, Object> tableProps = getTableUIProps();
		if (tableProps == null) {
			tableProps = new HashMap<String, Object>();
			setTableUIProps(tableProps);
		}
		HashMap<String, Object> colProps = (HashMap<String, Object>)tableProps.get(tableId);
		if (colProps == null) {
			colProps = new HashMap<String, Object>();
			tableProps.put(tableId, colProps);
		}
		TableColumn c;
		ArrayList<String> columnOrder = new ArrayList<String>();
		for (int colNumber = 0; colNumber < getColumnModel().getColumnCount(); colNumber++) {
			c = getColumnModel().getColumn(colNumber);
			columnOrder.add(getModel().getColumnName(c.getModelIndex()));
			colProps.put("colWidth" + colNumber, c.getWidth());
			//Log.log("Width for column " + colNumber + " stored: " + c.getWidth());
		}
		colProps.put("colOrderList", columnOrder);
		if (getRowSorter() != null && getRowSorter().getSortKeys() != null)
			colProps.put("sortKeyList", getRowSorter().getSortKeys());
	}

	@SuppressWarnings("unchecked")
	protected void applyUIProps() {

		HashMap<String, Object> tableProps = getTableUIProps();
		if (tableProps == null) {
			setDefaultUIProps();
			return;
		}
		HashMap<String, Object> colProps = (HashMap<String, Object>)tableProps.get(tableId);
		if (colProps == null) {
			setDefaultUIProps();
			return;
		}
		TableColumn c;
		Integer width;
		ArrayList<String> columnOrder = (ArrayList<String>)colProps.get("colOrderList");
		if (columnOrder != null) {
			ArrayList<TableColumn> columns = new ArrayList<TableColumn>();
			for (int colNumber = 0; colNumber < getColumnModel().getColumnCount(); colNumber++) {
				c = getColumnModel().getColumn(colNumber);
				c.setIdentifier(getModel().getColumnName(c.getModelIndex()));
				columns.add(c);
			}
			for (int colNumber = 0; colNumber < columnOrder.size(); colNumber++) {
				String colID = columnOrder.get(colNumber);
				int currentPos = -1;
				try {
					currentPos = getColumnModel().getColumnIndex(colID);
				} catch (Exception e) {}
				if (currentPos > -1) {
					if (currentPos != colNumber) {
						getColumnModel().moveColumn(currentPos, colNumber);
						//Log.log("Column moved: " + colID + " (from " + currentPos + " to " + colNumber);
					} else {
						//Log.log("Not moved: " + colID);
					}
				}
			}
		}
		for (int colNumber = 0; colNumber < getColumnModel().getColumnCount(); colNumber++) {
			c = getColumnModel().getColumn(colNumber);
			width = (Integer)colProps.get("colWidth" + colNumber);
			if (width != null) {
				c.setPreferredWidth(width);
				//Log.log("Width for column " + colNumber + " applied: " + c.getWidth());
			}
		}
		List<SortKey> skl = (List<SortKey>) colProps.get("sortKeyList");
		if (skl != null)
			getRowSorter().setSortKeys(skl);
	}

	/**
	 * Overload to set default column width etc.
	 */
	protected void setDefaultUIProps() {
	}
	
	public void dispose() {
		storeUIProps();
	}
	
	/**
	 * Utility function for toggling the sortOrder, but
	 * only if the given column-indexes match that of the
	 * current active sort columns.
	 * @param sortColumnNotToggled If -1, ignored else sort order for this column is not toggled.
	 * @param fileTableColumns
	 * @return True if sort order was toggled, false otherwise.
	 */
	@SuppressWarnings("unchecked")
	public boolean toggleSortOrder(final int sortColumnNotToggled, final int...fileTableColumns) {
		
		boolean alreadySet = false;
		List<SortKey> currentSkl = (List<SortKey>) getRowSorter().getSortKeys();
		Iterator<SortKey> it = currentSkl.iterator();
		int amountFound = 0;
		int c ;
		while (it.hasNext()) {
			c = it.next().getColumn();
			for (int fc : fileTableColumns)
				if (fc == c) amountFound++;
		}
		if (amountFound >= fileTableColumns.length) {
			alreadySet = true;
			it = currentSkl.iterator();
			ArrayList<SortKey> newSlk = new ArrayList<SortKey>(); 
			while (it.hasNext()) {
				SortKey sk = it.next();
				if (sortColumnNotToggled == sk.getColumn()) {
					newSlk.add(sk);
				} else {
					SortKey toggled = new SortKey(sk.getColumn(), 
							(sk.getSortOrder() == SortOrder.ASCENDING ? SortOrder.DESCENDING : SortOrder.ASCENDING));
					newSlk.add(toggled);
				}
			}
			getRowSorter().setSortKeys(newSlk);
			
		}
		//Log.log("Sort order " + (alreadySet ? "not" : "") + " toggled.");
		return alreadySet;
	}
}
