package net.java.dev.fwutil;

import java.awt.Component;
import java.awt.ComponentOrientation;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;

import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;

/** An example class that makes numbers more readable in a table. */
public class TableCellNumberRenderer extends DefaultTableCellRenderer {

	private static final long serialVersionUID = 2121024499022643500L;
	
	public static DecimalFormat numberFormat = getNumberFormat();
	static {
		numberFormat.setMaximumFractionDigits(3);
	}

	public TableCellNumberRenderer() {
		super();
		if (getComponentOrientation().isLeftToRight()) {
			applyComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
		} else {
			applyComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
		}
	}
	
	/** Returns an instance of the DecimalFormat that this number formatter uses.
	 * Returned with default settings.
	 */
	public static DecimalFormat getNumberFormat() {
		
		DecimalFormat nf = new DecimalFormat();
		nf.setGroupingUsed(true);
		nf.setGroupingSize(3);
		nf.setParseBigDecimal(true);
		nf.setParseIntegerOnly(false);
		nf.setDecimalSeparatorAlwaysShown(false);
		DecimalFormatSymbols dfs = nf.getDecimalFormatSymbols();
		dfs.setDecimalSeparator(',');
		dfs.setGroupingSeparator(' ');
		dfs.setNaN("NaN");
		nf.setDecimalFormatSymbols(dfs);
		return nf;
	}
	
	public static String getTextValue(final Object value, final DecimalFormat formatter) {
		
		String s;
		if (value == null) {
			s = "";
		} else if (value instanceof String) {
			s = value.toString();
		} else {
			if (formatter.getMaximumFractionDigits() > 0 && value instanceof Double) {
				if (-10.1 < (Double)value && (Double)value < 10.1) {
					formatter.setMaximumFractionDigits(2);
				} else if ((Double)value > 1.0) {
					formatter.setMaximumFractionDigits(1);
				}
			}
			try {
				s = formatter.format(value) + Util.space;
			} catch (Exception nan) {
				Log.logError("Invalid number: " + value.getClass().getCanonicalName());
				s = "NaN";
			}
		}
		return s;
	}

	public Component getTableCellRendererComponent(final JTable table, final Object value,
			final boolean isSelected, final boolean hasFocus, final int rowIndex, final int colIndex) {

		if (isSelected || hasFocus) {
			setBackground(table.getSelectionBackground());
		} else {
			setBackground(table.getBackground());
		}
		setText(getTextValue(value, numberFormat));
		return this;
	}
}
