package net.java.dev.fwutil.gui;

import java.awt.Component;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import net.java.dev.fwutil.Labels;
import net.java.dev.fwutil.LogUI;

/** Base Window menu with New Window, Close Window and Show log.
 * @author Fred
 *
 */
public class BMWindow extends JMenu implements ActionListener {

	private static final long serialVersionUID = 6110242369587664493L;
	
	protected IMainWindow mainWindow;

	public BMWindow(IMainWindow mainWindow) {
		super(Labels.get("Window"));
		this.mainWindow = mainWindow;
		setMnemonic(KeyEvent.VK_W);
		addMenuItems();
	}
	
	/** Override to add your own menu-items. */
	public void addMenuItems() {

		JMenuItem m;
		m = new JMenuItem(Labels.get(BMLabel.NewWindow));
		m.setMnemonic(KeyEvent.VK_N);
		m.setActionCommand(BMLabel.NewWindow);
		m.addActionListener(this);
		add(m);
		m = new JMenuItem(Labels.get(BMLabel.CloseWindow));
		m.setMnemonic(KeyEvent.VK_C);
		m.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, KeyEvent.ALT_DOWN_MASK));
		m.setActionCommand(BMLabel.CloseWindow);
		m.addActionListener(this);
		add(m);
		
		addSeparator();
		m = new JMenuItem(Labels.get(BMLabel.ShowLog));
		m.setMnemonic(KeyEvent.VK_L);
		m.setActionCommand(BMLabel.ShowLog);
		m.addActionListener(this);
		add(m);
	}
	
	/** Override to catch your own action events from menu items. */
	public void actionPerformed(ActionEvent ae) {
		
		final String ac = ae.getActionCommand();
		if (BMLabel.NewWindow == ac) {
			// By passing mainWindow as component, the new MainWindow will appear without completely overlapping
			// the current window (shows a little down and to the right)  
			BMainWindow.openWindow(mainWindow.getClass(), mainWindow.getMainRestart(), (Component)mainWindow);
		} else if (BMLabel.CloseWindow == ac) {
			if (LogUI.openChildWindows() == 1) {
				mainWindow.exitApp(false);
			} else {
				if (mainWindow.closeAllowed()) {
					this.getToolkit().getSystemEventQueue().postEvent(new WindowEvent((Window)mainWindow, WindowEvent.WINDOW_CLOSING));
				}
			}
		} else if (BMLabel.ShowLog == ac) {
			LogUI.makeVisible();
		}
	}
}
