/*
 * This software is licensed under the GNU Lesser GPL, see
 * http://www.gnu.org/licenses/ for details.
 * You can use this free software in commercial and non-commercial products,
 * as long as you give credit to this software when you use it.
 * This softwware is provided "as is" and the use of this software is 
 * completely at your own risk, there are absolutely no warranties.
 * 
 */

package net.java.dev.hvlayout;


import java.awt.Dimension;
import java.awt.event.MouseEvent;

import javax.swing.JToolTip;
import javax.swing.JTextArea;
import javax.swing.JViewport;

/**
 * HVTextArea is used to show multi-line labels and overloads the usual JTextArea
 * to return the correct sizes so that the text area behaves properly with HVLayout.  
 * @author Fred
 *
 */
public class HVTextArea extends JTextArea {
	
	private static final long serialVersionUID = 1L;
	
	public HVTextArea(String text) {
		super(text);
	}
	
	/**
	 * Overload to use JMultiLineToolTip which shows text just
	 * like text area does.
	 */
	public JToolTip createToolTip()	{
		return new JMultiLineToolTip(getFont());
	}
	
	/**
	 * Only show a tooltip when the text area is not fully shown
	 * AND the text area is not inside a scroller (because then you
	 * can scroll to see the text, a tooltip is then only annoying).
	 * 
	 */
	public String getToolTipText(final MouseEvent e) {
		if (!(getParent() instanceof JViewport))
			return (getVisibleRect().width < getPreferredSize().width ? getText() : null);
		return null;
	}
	
	/**
	 * Return proper minimumsize for HVLayout.
	 * Note that preferred size should NOT be touched,
	 * this is internally calculated by text area and needed by HVLayout.
	 */
	public Dimension getMinimumSize() {
		final Dimension d = super.getMinimumSize();
		d.height = getPreferredSize().height;
		d.width = HVProps.defaultLineWidth;
		return d;
	}

	/**
	 * Return proper maximumsize for HVLayout.
	 * Note that preferred size should NOT be touched,
	 * this is internally calculated by text area and needed by HVLayout.
	 */
	public Dimension getMaximumSize() {
		final Dimension d = super.getMaximumSize();
		d.height = getPreferredSize().height;
		return d;
	}
}
