package net.java.dev.fwutil.pf;

import java.io.File;
import java.io.Serializable;

import javax.swing.Icon;

//import net.java.dev.fwutil.Log;

public class FileInfo implements Serializable, Comparable<FileInfo> {
	
	private static final long serialVersionUID = -9145521882401653464L;

	public String name = null;
	public long modificationDate = 0;
	public boolean isDirectory = false;
	public long size = 0;
	public boolean isHidden = false;
	public boolean isReadOnly = false;
	public Icon icon = null;

	public FileInfo() {
		super();
	}
	
	public FileInfo(final File f) {
		super();
		setFileInfo(f);
	}
	
	public FileInfo(final String path) {
		super();
		setFileInfo(new File(path));
	}
	
	public FileInfo(final String path, boolean isRoot) {
		super();
		if (isRoot) {
			name = path;
			modificationDate = System.currentTimeMillis();
			isDirectory = true;
		} else {
			setFileInfo(new File(path));
		}
	}
	
	public void setFileInfo(final File f) { 
		
		//Log.log("Adding FileInfo from File " + f.getAbsolutePath());
		name = f.getName();
		if (f.exists()) {
			modificationDate = f.lastModified();
			isDirectory = f.isDirectory();
			isHidden = f.isHidden();
			isReadOnly = !f.canWrite();
			if (!isDirectory) size = f.length();
		}
	}
	
	@Override
	public String toString() {
		return name;
	}

	public int compareTo(FileInfo other) {
		
		if (name == null && other.name == null) {
			return 0;
		}
		if (name != null) {
			return (other.name == null ? name.compareTo(null) : name.toUpperCase().compareTo(other.name.toUpperCase()));
		}
		// name is null and value is not.
		return -1;
	}
}
