package net.java.dev.fwutil.pf;

import java.awt.Font;

import net.java.dev.fwutil.Labels;
import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.LogUI;
import net.java.dev.fwutil.Restartable;
import net.java.dev.fwutil.Settings;
import net.java.dev.fwutil.Startup;
import net.java.dev.fwutil.UIMessages;
import net.java.dev.fwutil.pf.ui.MainWindow;
import net.java.dev.hvlayout.HVProps;

public class PathFinder implements Restartable {

	// Needed in case of restart - MainFrame passes this Restartable on to
	// StartUp.close(Restartable);
	private static final PathFinder thisInstance = new PathFinder();
	
	private PathFinder() {
		super();
	}
	
	public static PathFinder getInstance() {
		return thisInstance;
	}

	/** The restart method as defined by the Restartable interface. 
	 * Simply calls main again.
	 */
	public void restart(String[] args) {
		main(args);
	}
	
	public static void main(String[] args) {

		Settings.appName = "PathFinder";
		Settings.appVersion = "0.2";
		Settings.defaultSettingsFile = "PathFinderSettings.xml";
		Labels.labelFile = "resources/labels.txt";
		if (!Startup.boot(args, Settings.appName, Settings.appVersion, ".PathFinder")) {
			Log.log("Startup failed, exiting.");
			UIMessages.showErrorMessage(LogUI.getInstance(), "StartUp error", "Error during startup, please examine log.\nClosing " + Settings.appName);
			Log.close();
			System.exit(1);
		}
		Log.log(Labels.get("MainFrameStart"));
		MainWindow.openWindow(MainWindow.class, thisInstance, null);
	}

	/** Returns An instance of a mono-spaced font (like Courier) with the
	 * default font size.
	 */
	public static Font getDefaultUIFontMonoSpaced() {
		return new Font("Monospaced", 0, HVProps.getDefaultFontSize());
	}
}
