package net.java.dev.fwutil.pf.ui;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.KeyStroke;
import javax.swing.WindowConstants;

import net.java.dev.fwutil.KeyAction;
import net.java.dev.fwutil.Labels;
import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.Settings;
import net.java.dev.fwutil.WindowCloseAdapter;
import net.java.dev.fwutil.WindowDimension;
import net.java.dev.fwutil.pf.ui.menu.MBookmarks;
import net.java.dev.hvlayout.FormUI;
import net.java.dev.hvlayout.HBox;
import net.java.dev.hvlayout.VBox;

public class BookmarkDialog extends JDialog implements ActionListener {

	private static final long serialVersionUID = -6749289136167771786L;

	private MBookmarks mbookmarks;
	private String bookmark;
	private JButton bCancel;
	private JButton[] bBookmarks;

	private BookmarkDialog(Frame owner, String bookmark, MBookmarks mbookmarks) {
		super(owner, Labels.get("SetBookMark"), true);
		this.bookmark = bookmark;
		this.mbookmarks = mbookmarks;
		setModalityType(ModalityType.APPLICATION_MODAL);
		init();
		buildPane();
	}
	
	private void init() {

		bBookmarks = new JButton[9];
    	bCancel = FormUI.getButton(this, Labels.get("Cancel"));
		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowCloseAdapter(bCancel, this));
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), bCancel);
		getRootPane().getActionMap().put(bCancel, new KeyAction(bCancel, this));
	}
	
	private void buildPane() {
		
		VBox mainPane = new VBox();
		mainPane.add(FormUI.getLabel(Labels.get("OverwriteBookmark"), 0));
		mainPane.add(FormUI.getLabel(bookmark, 0));
		for (int i = 0; i < bBookmarks.length; i++) {
			HBox hb = new HBox();
			JButton b = FormUI.getSmallButton(Integer.toString(i + 1));
			b.addActionListener(this);
			bBookmarks[i] = b;
			hb.add(b);
			hb.add(FormUI.getLabel((String)Settings.get("PFBookmark" + (i + 1)), 0));
			mainPane.add(hb);
		}
		HBox hb = new HBox(HBox.OPPOSITELINEAXIS);
		hb.add(bCancel);
		mainPane.add(hb);
		setContentPane(mainPane);
	}

	
	public static void show(Frame owner, String bookMark, MBookmarks mbookmarks) {
		
		BookmarkDialog dialog = new BookmarkDialog(owner, bookMark, mbookmarks);
		dialog.pack();
		WindowDimension.applyWindowDimension(dialog.getTitle(), dialog, owner);
		dialog.setVisible(true);
		dialog.dispose();
	}

	public void actionPerformed(ActionEvent action) {
		
		Object as = action.getSource();

		if (as == bCancel) {
			setVisible(false);
		} else {
			int bmNumber = -1;
			int i = 0;
			while (bmNumber == -1 && i < bBookmarks.length) {
				if (action.getSource() == bBookmarks[i]) bmNumber = i + 1;
				i++;
			}
			if (bmNumber != -1) {
				Settings.put("PFBookmark" + (bmNumber), bookmark);
				Log.log("Bookmark " + bmNumber + " set to: " + bookmark);
				mbookmarks.updateBookmarks();
				setVisible(false);
			}
		}
	}
}
