package net.java.dev.fwutil.pf.ui;

import java.awt.Color;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.util.Date;
import java.util.HashMap;

import javax.swing.JButton;
import javax.swing.JLabel;
//import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.TableCellNumberRenderer;
import net.java.dev.fwutil.Util;
import net.java.dev.fwutil.pf.FileInfo;
import net.java.dev.fwutil.pf.ui.table.FileTableColumns;
import net.java.dev.hvlayout.FormUI;
import net.java.dev.hvlayout.HBox;
import net.java.dev.hvlayout.HVProps;
import net.java.dev.hvlayout.VBox;
import java.awt.datatransfer.StringSelection;

public class FileDetailsPanel extends VBox implements ListSelectionListener, ActionListener, ClipboardOwner {
	
	private static final long serialVersionUID = -3925215954547029801L;

	private BrowsePanel browsePanel;
	private HashMap<FileTableColumns, JButton> valueButtons;
	private HashMap<FileTableColumns, JLabel>  fileValues;
	
	public FileDetailsPanel(final BrowsePanel browsePanel) {
		super();
		this.browsePanel = browsePanel;
		valueButtons = new HashMap<FileTableColumns, JButton>();
		fileValues = new HashMap<FileTableColumns, JLabel>();
		browsePanel.getTable().getSelectionModel().addListSelectionListener(this);
		add(getHBox("Name", FileTableColumns.Name));
		HBox top = new HBox();
		VBox left = new VBox();
		left.add(getHBox("Size", FileTableColumns.SizeBytes));
		left.add(getHBox("Modified", FileTableColumns.Modified));
		top.add(left);
		VBox middle = new VBox();
		middle.add(getHBox("Extension", FileTableColumns.Extension));
		middle.add(getHBox("Attributes", FileTableColumns.Hidden));
		top.add(middle);
		add(top);
	}
	
	private HBox getHBox(final String label, final FileTableColumns fp) {
		
		HBox hb = new HBox();
		JButton valueButton = FormUI.getButton(this, label);
		valueButtons.put(fp, valueButton);
		hb.add(valueButtons.get(fp));
		boolean oppositeLineOut = (fp == FileTableColumns.Modified || fp == FileTableColumns.SizeBytes);
		final JLabel lui = FormUI.getLabel(label, HVProps.defaultButtonWidth, oppositeLineOut, false);
		fileValues.put(fp, lui);
		lui.setOpaque(true);
		lui.setBackground(Color.WHITE);
		//fileValues[labelId].setEditable(false);
		lui.setEnabled(true);
		hb.add(lui);
		return hb;
	}

	public void valueChanged(ListSelectionEvent e) {
		
		if (e.getValueIsAdjusting()) return;
		int rowIndex = browsePanel.getTable().getSelectedRow();
		if (rowIndex > -1) {
			FileInfo file = browsePanel.getTable().getFileInfo(rowIndex);
			if (file != null) {
				JLabel lui = fileValues.get(FileTableColumns.Name); 
				lui.setIcon(file.icon);
				lui.setText(file.name);
				lui = fileValues.get(FileTableColumns.Modified); 
				lui.setText(DateFormat.getDateTimeInstance(DateFormat.FULL, DateFormat.LONG).format(new Date(file.modificationDate)));
				lui = fileValues.get(FileTableColumns.SizeBytes); 
				lui.setText(TableCellNumberRenderer.numberFormat.format(file.size));
				lui = fileValues.get(FileTableColumns.Extension); 
				lui.setText(FileTableColumns.Extension.getTableValue(file).toString());
				String att = Util.emptyString;
				if (file.isHidden) att="hidden";
				if (file.isReadOnly) {
					if (att == Util.emptyString) { att="read-only"; }
					else { att = att + "; read-only"; }
				}
				lui = fileValues.get(FileTableColumns.Hidden); 
				lui.setText(att);
			}
		}
	}
	
	public void actionPerformed(final ActionEvent e) {
		
		Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
		final FileInfo fi = browsePanel.getTable().getFileInfo(browsePanel.getTable().getSelectedRow());
		String copyToClipboard = Util.emptyString;
		if (e.getSource() == valueButtons.get(FileTableColumns.Name)) {
			copyToClipboard = browsePanel.getLocation().getLastPathKey() + fi.name;
		} else if (e.getSource() == valueButtons.get(FileTableColumns.Modified)) {
			copyToClipboard = (new Date(fi.modificationDate)).toString();
		} else if (e.getSource() == valueButtons.get(FileTableColumns.SizeBytes)) {
			copyToClipboard = Long.toString(fi.size);
		} else if (e.getSource() == valueButtons.get(FileTableColumns.Hidden)) {
			if (fi.isHidden) copyToClipboard = copyToClipboard + "H";
			if (fi.isReadOnly) copyToClipboard = copyToClipboard + "R";
		} else if (e.getSource() == valueButtons.get(FileTableColumns.Extension)) {
			copyToClipboard = FileTableColumns.Extension.getTableValue(fi).toString();
		}
		Log.log("Copied to clipboard: " + copyToClipboard);
		clipboard.setContents(new StringSelection(copyToClipboard), this);
	}

	public void lostOwnership(final Clipboard arg0, final Transferable arg1) {
		// do nothing
	}
}
