package net.java.dev.fwutil.pf.ui.table;

import java.util.Date;

import net.java.dev.fwutil.Labels;
import net.java.dev.fwutil.Util;
import net.java.dev.fwutil.pf.FileInfo;
import net.java.dev.hvlayout.HVProps;

public enum FileTableColumns {

	Type(String.class, HVProps.defaultLineHeight), 
	Name(FileInfo.class, (int)(HVProps.defaultLineWidth * 1.5)), 
	Extension(String.class, HVProps.defaultLineHeight * 2), 
	Size(Long.class, HVProps.defaultButtonWidth),
	Modified(Date.class, (int)(HVProps.defaultButtonWidth * 1.5)),
	Hidden(String.class, HVProps.defaultLineHeight),
	ReadOnly(String.class, HVProps.defaultLineHeight),
	SizeBytes(Long.class, HVProps.defaultButtonWidth);
	
	public static final char ExtensionSeparator = '.';
	
	public Class<?> tableClass;
	public int defaultWidth;
	
	private FileTableColumns(Class<?> tableClass, int defaultWidth) {
		this.tableClass = tableClass;
		this.defaultWidth = defaultWidth;
	}
	
	public String getCName() {
		return Labels.get("C"+this.name());
	}
	
	public Object getTableValue(FileInfo f) {
		
		if (this == Type) {
			return (f.isDirectory ? "+" : Util.emptyString);
		} else if (this == Name) {
			return f.name;
		} else if (this == Modified) {
			return new Date(f.modificationDate);
		} else if (this == Hidden) {
			return (f.isHidden ? "H" : Util.emptyString);
		} else if (this == ReadOnly) {
			return (f.isReadOnly ? "R" : Util.emptyString);
		} else if (this == Size || this == SizeBytes) {
			return f.size;
		} else if (this == Extension) {
			if (f.isDirectory) return Util.emptyString;
			final int lastDot = f.name.lastIndexOf(ExtensionSeparator);
			if (lastDot < 2 || lastDot >= f.name.length()-1) return Util.emptyString;
			return f.name.substring(lastDot + 1, f.name.length());
		}
		return Util.emptyString;
	}
}