package net.java.dev.fwutil.pf.ui.table;

import java.util.ArrayList;
import java.util.List;

import javax.swing.table.AbstractTableModel;

import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.Util;
import net.java.dev.fwutil.pf.FileInfo;

public class FileTableDataModel extends AbstractTableModel {

	private static final long serialVersionUID = 4408770778979062862L;

	protected List<FileInfo> activeFileList;
	
	public FileTableDataModel() {
		super();
		// activeFileList cannot be null.
		activeFileList = new ArrayList<FileInfo>();
	}
	
	public void setFiles(List<FileInfo> files) {
		
		activeFileList = files;
		fireTableDataChanged();
	}

	@Override
	public Class<?> getColumnClass(final int c) {
		
		/**
		 * from
		 * http://java.sun.com/docs/books/jls/third_edition/html/classes.html#8.9
		* ...if E is the name of an enum type, then that type has
		* the following implicitly declared static methods:
		* Returns an array containing the constants of this enum
		* type, in the order they're declared.  This method may be
		* used to iterate over the constants as follows:
		*
		*    for(E c : E.values())
		*        System.out.println(c);
		*
		* @return an array containing the constants of this enum
		* type, in the order they're declared
		* 
		* public static E[] values();
		*/
		return FileTableColumns.values()[c].tableClass;
		/*
		FileColumns[] fc = FileColumns.values();
		if (c > -1 && c < fc.length)
			return fc[c].tableClass;
		Log.logError("Invalid column index for class: " + c);
		return super.getColumnClass(c);
		*/
	}

	public int getColumnCount() {
		return FileTableColumns.values().length;
	}

	@Override
	public String getColumnName(final int c) {
		return FileTableColumns.values()[c].getCName();
	}

	public int getRowCount() {
		return activeFileList.size();
	}
	
	public FileInfo getRow(final int row) {
		return (row > -1 && row < getRowCount() ? activeFileList.get(row) : null);
	}

	public Object getValueAt(final int r, final int c) {
		
		if (r > -1 && r < activeFileList.size()) {
			return FileTableColumns.values()[c].getTableValue(activeFileList.get(r));
			/*
			FileColumns[] fc = FileColumns.values();
			if (c > -1 && c < fc.length)
				return fc[c].getTableValue(activeFileList.get(r));
			Log.logError("Invalid column index for value: " + c);
			*/
		}
		Log.log("Invalid row index for value: " + c);
		return Util.emptyString;
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}
}
