package net.java.dev.fwutil.pf.ui.table;

import java.awt.Component;
import java.io.File;
import java.util.HashMap;

import javax.swing.Icon;
import javax.swing.JFileChooser;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;

import net.java.dev.fwutil.QTask;
import net.java.dev.fwutil.TaskQueue;
import net.java.dev.fwutil.Util;
import net.java.dev.fwutil.pf.DiskInfo;
import net.java.dev.fwutil.pf.FileInfo;

public class FileTypeCellrenderer extends DefaultTableCellRenderer {

	private static final long serialVersionUID = 6203297160974447330L;
	
	private static final TaskQueue tq = new TaskQueue();
	
	private static final FileTypeCellrenderer thisInstance = new FileTypeCellrenderer();

	@Override
	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, 
			boolean hasFocus, int row, int column) {
		
        if (isSelected || hasFocus) {
			setBackground(table.getSelectionBackground());
		} else {
			setBackground(table.getBackground());
		}
		setText(Util.emptyString);
		setIcon(getIcon(table, row));
        return this;
    }
	
	public static final HashMap<String, Icon> icons = new HashMap<String, Icon>();
	protected static final JFileChooser chooser = new JFileChooser();

	public static Icon getIcon(final JTable table, final int row) {

		final FileInfo fi = ((FileTable)table).getFileInfo(row);
		String extension;
		if (fi.isDirectory) { 
			extension = null;
		} else {
			extension = (String) FileTableColumns.Extension.getTableValue(fi);
		}
		return getFileIcon((FileTable) table, extension, row, fi);
	}
	
	public static Icon getFileIcon(final FileTable table, final String extension, 
			final int row, final FileInfo fi) {
		
		if (fi.icon != null) return fi.icon;
		if (extension == null
				|| (DiskInfo.windowsOS 
						&& (extension.equalsIgnoreCase("exe")) 
							|| extension.equalsIgnoreCase("com"))
							|| extension.equalsIgnoreCase("lnk")) {
			fi.icon = icons.get(table.getFilePath(row));
			if (fi.icon == null) {
				tq.add(thisInstance.new IconLookUp(table, table.getFilePath(row), row));
			}
		} else {
			fi.icon = icons.get(extension);
			if (fi.icon == null) {
				tq.add(thisInstance.new IconLookUp(table, extension, row)); 
			}
		}
		return fi.icon;
	}
	
	class IconLookUp extends QTask {
		
		private final FileTable table;
		private final String key;
		private final File file;
		private final FileInfo fi;
		
		public IconLookUp(final FileTable table, final String key, final int row) {
			super(table.getLastPathKey());
			this.table = table;
			this.key = key;
			this.file = new File(table.getLastPathKey() + table.getFileInfo(row).name);
			this.fi = table.getFileInfo(row); 
		}
		
		public void run() {
			// The chooser can take a long time to return the icon for the file.
			fi.icon = chooser.getIcon(file);
			icons.put(key, fi.icon);
			// Check if table still displays same directory
			if (table.getLastPathKey().equals(getBatchId()))
				table.updateFileUI(fi);
			else 
				tq.clear(getBatchId());
		}
	}
}
