package nl.intercommit.mario.tools;

import javax.jms.TextMessage;
import javax.management.MBeanServerConnection;
import javax.naming.InitialContext;

import org.apache.log4j.Logger;

/*
import javax.management.MBeanServerConnection;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
 */

/**
 * Following libraries are required:
 * - for InitialContext: jboss/client jbossall-client.jar
 * - for ConnectionFactory:
 *    - from jboss\client:
 *    javassis.jar, jboss-aop-jdk50-client.jar, trove.jar and log4j.jar
 *    - from jboss/server/all/deploy/jboss-aop-jdk50.deployer
 *    jboss-aop-jdk50.jar
 *    - from Jboss-messaging distribution package
 *    jboss-messaging-client.jar
 * @author frederikw
 *
 */
public class MessageTest {

	private static Logger log = null;

	/**
	 * @param args
	 */
	public static void main(final String[] args) {
		ConfUtil.loadLog4j();
		log = Logger.getLogger(MessageTest.class);
		//connectXML();
		doTest();
	}

	private static void doTest() {

		//ConfUtil.setSecurity(null);
		InitialContext ic = JmsUtil.getContext();
		MBeanServerConnection mserver = JmsUtil.getMBeanServer(ic);
		try {
			JmsUtil.enableQueueStats(mserver, true);
			log.info("Listing domains:");
			String [] doms = JmsUtil.getDomainNames(mserver);
			String domsText = doms[0];
			for(int i = 1; i < doms.length; i++)
				domsText = domsText + ", " + doms[i];
			log.info(domsText);
			log.info("Listing queue names:");
			log.info(JmsUtil.getQueueNames(ic));
			JmsUtil.deployQueue(ic, "A");
			JmsUtil.drainQueue(ic, "A", 1000);
			for (int i = 0; i < 10; i++)
				JmsUtil.sendMessage(ic, "A", "Hello " + i);
			log.info("Listing attributes of queue A:");
			log.info(JmsUtil.getQueueAttValues(mserver, "A"));
			log.info("Expecting 10 messages on queue A:");
			JmsUtil.printMessageList(JmsUtil.getMessages(ic, "A", null, null));
			TextMessage tm = (TextMessage) JmsUtil.receiveMessage(ic, "A");
			if (tm != null && tm.getText().equals("Hello 0"))
				log.info("Message 1 received OK");
			else
				log.error("Did not receive the expected message \"Hello 0\": " + tm.getText());
			tm = (TextMessage) JmsUtil.receiveMessage(ic, "A");
			if (tm != null && tm.getText().equals("Hello 1"))
				log.info("Message 2 received OK");
			else
				log.error("Did not receive the expected message \"Hello 1\": " + tm.getText());
			JmsUtil.drainQueue(ic, "A", 8);
			// Destroy queue does not appear to work.
			//JmsUtil.destroyQueue(ic, "A");
		} catch (Exception e) {
			log.info("Error during test:", e);
		} finally {
			JmsUtil.closeContext(ic);
		}
		log.info("Done.");
	}
}

