package nl.intercommit.mario.tools.jbossmon;

import java.beans.XMLEncoder;
import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;

import javax.swing.ImageIcon;

import net.java.dev.fwutil.Labels;
import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.LogUI;
import net.java.dev.fwutil.Settings;
import net.java.dev.fwutil.SortKeyPersistenceDelegate;
import net.java.dev.fwutil.Startup;
import net.java.dev.fwutil.UIMessages;
import net.java.dev.hvlayout.HVProps;
import nl.intercommit.mario.tools.ConfUtil;

public class ICITMon {

	public static final String labelsFile = "resources/config/mariotoolslabels.txt";
	public static final String ICITLogoFile = "resources/501.png";
	public static ImageIcon ICITLogo  = null;
	private static Properties jndiProps;
	
	static {
		// Following two statements allow SortKery values to be stored in Settings.
		// Only needs to be done once.
		XMLEncoder encoder = new XMLEncoder(System.out);
		encoder.setPersistenceDelegate(javax.swing.RowSorter.SortKey.class, new SortKeyPersistenceDelegate());
	}
	
	public static void main(final String[] args) {

		Settings.appName = "ICIT JMS Monitor";
		Settings.appVersion =  "0.1";
		Settings.defaultSettingsFile = "ICITMonSettings" + Settings.appVersion + ".xml";
		Labels.labelFile = labelsFile;
		Log.logFileName = new String[] {"icitmon0.log", "icitmon1.log", "icitmon2.log"};
		if (!Startup.boot(args, Settings.appName, Settings.appVersion, null)) {
			exitOnError();
		}
		ConfUtil.loadLog4j();
		jndiProps = ConfUtil.loadJNDI();
		if (jndiProps == null) {
			Log.logError("Could not get JNDI properties.");
			exitOnError();
		}
		ICITLogo = loadIcon(ICITLogoFile);
		// The default font is very small.
		if (Settings.get(Settings.defaultFontSize) == null) {
            Settings.put(Settings.defaultFontSize, 12);
            HVProps.setDefaultUIFontSize(12);
		}
		// StartUp went OK, start the main Window.
		Log.log("Starting connect window");
		// Use SwingUtilities.invokeLater to prevent rare occasions where
		// new JFrames simply do not appear.
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				// Prevent closing the log from closing the application.
				LogUI.setProgrammaticClose();
				JbossConnect.showWindow();
			}
		});
	}

	public static void exitOnError() {

		Log.log("Startup failed, exiting.");
		UIMessages.showErrorMessage(LogUI.getInstance(), "StartUp error", "Error during startup, please examine log.\nClosing " + Settings.appName);
		Log.close();
		System.exit(1);
	}

	private static ImageIcon loadIcon(final String fname) {

		ImageIcon iicon = null;
		File marioFile = new File(fname);
		if (marioFile.exists() && !marioFile.isDirectory()) {
			FileInputStream fin = null;
			try {
				fin = new FileInputStream(marioFile);
				byte[] imageData = new byte[(int) marioFile.length()];
				fin.read(imageData, 0, (int) marioFile.length());
				iicon = new ImageIcon(imageData, fname);
			} catch (Exception e) {
			} finally {
				if (fin != null) try { fin.close(); } catch (Exception e) {;}
			}
		}
		return iicon;
	}
}
