package nl.intercommit.mario.tools.jbossmon;

import java.awt.Container;

import javax.swing.JLabel;

import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.MFrame;
import net.java.dev.fwutil.Settings;
import net.java.dev.fwutil.UIMessages;
import net.java.dev.hvlayout.FormUI;
import net.java.dev.hvlayout.HVProps;
import net.java.dev.hvlayout.VLayout;
import nl.intercommit.mario.tools.JmsUtil;
import nl.intercommit.mario.tools.MarioContext;

public class JbServerMon extends MFrame {

	private static final long serialVersionUID = -8544409455515275063L;
	private final ServerQueueTable qTable;
	protected String serverURL;
	private JLabel status;
	private MarioContext mc;

	/** For use with dummy test.*/
	protected JbServerMon(final String title) {
		super(null, title);
		qTable = null;
		display(null);
	}

	public JbServerMon(final JbossConnect pw, final MarioContext mc, final String serverURL, final String windowTitle) {
		super(pw, Settings.appName + " " + windowTitle);
		if (ICITMon.ICITLogo != null)
			setIconImage(ICITMon.ICITLogo.getImage());
		this.serverURL = serverURL;
		this.mc = mc;
		Container mainPanel = getContentPane();
		mainPanel.setLayout(new VLayout());
		// Must set status before qmon, qmon uses status.
		status = FormUI.getLabel("Initializing ...", 2 * HVProps.defaultLineWidth);
		qTable = new ServerQueueTable(mc, this);
		mainPanel.add(HVProps.getScroller(qTable, 3 * HVProps.defaultLineWidth, 20, false));
		mainPanel.add(status);
		setJMenuBar(new ServerMonMenu(this).getJMenuBar());
		display(pw);
	}

	public String getServerURL() {
		return serverURL;
	}

	public void setStatus(final String statusInfo) {

		Log.log(serverURL + " - " + statusInfo);
		status.setText(statusInfo);
	}

	public void dispose() {

		if (qTable != null) qTable.dispose();
		if (mc != null) mc.close();
		super.dispose();
	}

	public void refreshQueueList() {
		qTable.getMonitor().refreshQueues();
	}

	public boolean queueIsSelected() {

		boolean qselected = (qTable.getSelectedQueue() != null);
		if (!qselected) {
			UIMessages.showErrorMessage(this, "No queue selected", "You have to select a queue first.");
		}
		return qselected;
	}

	/** Removes messages from queue after asking confirmation. */
	public void clearQueue() {

		if (!queueIsSelected())
			return;
		if (UIMessages.askYesNo(this, "Delete messages on queue", "Are you sure you want to delete the messages\n" +
				"on queue " + qTable.getSelectedQueue() + "?"));
		qTable.getMonitor().clearQueue(qTable.getSelectedQueue());
	}

	public void resetCounters() {
		qTable.getMonitor().resetCounters();
	}

	public ServerQueueTable getMonTable() {
		return qTable;
	}

	public void disableMsgStats(final boolean value) {
		qTable.getMonitor().disableMsgStats(value);
	}

	public void resetAverages() {
		qTable.getMonitor().resetStats();
	}

	public void openSendMsg() {

		if (!queueIsSelected())
			return;
		new SendMsg(this, qTable.getSelectedQueue());
	}

	public void deployQueue() {
		
		String queueName = UIMessages.askLabel(this, "Queue name", "The name of the new queue");
		if (queueName != null && queueName.length() > 0) {
			if (JmsUtil.deployQueue(mc.ic, queueName)) {
				setStatus("New queue created: " + queueName);
				refreshQueueList();
			} else {
				UIMessages.showErrorMessage(this, "Deploy queue", "Something went wrong deploying the new queue");
			}
		}
	}

	public void destroyQueue() {

		if (!queueIsSelected())
			return;
		boolean destroy = UIMessages.askYesNo(this, "Remove queue", "Are you sure you want to delete queue " + qTable.getSelectedQueue());
		if (destroy) {
			if (JmsUtil.destroyQueue(mc.ic, qTable.getSelectedQueue())) {
				UIMessages.showMessage(this, "Queue destroyed", "Queue has been destroyed");
				refreshQueueList();
			} else {
				UIMessages.showErrorMessage(this, "Queue removal", "Something went wrong removing the queue");
			}
		}
	}

	public void openQueueBrowser() {
		
		if (!queueIsSelected())
			return;
		new QBrowser(this, mc, qTable.getSelectedQueue());
	}
}
