package nl.intercommit.mario.tools.jbossmon;

import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Properties;

import javax.management.MBeanServerConnection;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;

import net.java.dev.fwutil.KeyAction;
import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.LogUI;
import net.java.dev.fwutil.MFrame;
import net.java.dev.fwutil.Settings;
import net.java.dev.fwutil.Startup;
import net.java.dev.fwutil.UIMessages;
import net.java.dev.hvlayout.FormUI;
import net.java.dev.hvlayout.HBox;
import net.java.dev.hvlayout.HVProps;
import net.java.dev.hvlayout.VLayout;
import nl.intercommit.mario.tools.ConfUtil;
import nl.intercommit.mario.tools.JmsUtil;
import nl.intercommit.mario.tools.MarioContext;

public final class JbossConnect extends MFrame implements ActionListener, MouseListener {

	public static final String jbServerProp = "java.naming.provider.url";
	private static final long serialVersionUID = 8335818027643573325L;
	private static Properties jndiProps;
	private final JTextField tfJbServer;
	private final JTextField tfUserName;
	private final JPasswordField pfPasswd;
	private final JCheckBox cbLoginAnonymous;
	private final String jbServer;
	private final JButton bConnect;
	private final JButton bExit;
	private final JButton bshowLog;
	private static ArrayList<JbServerInfo> serverList;
	private final JList lopenServers;
	private static DefaultListModel lmopenServers;
	private static final JbossConnect thisInstance = new JbossConnect();

	private JbossConnect() {
		super(LogUI.getInstance(), Settings.appName + " " + Settings.appVersion + " " + "Connect to JBoss server");
		if (ICITMon.ICITLogo != null)
			setIconImage(ICITMon.ICITLogo.getImage());
		jndiProps = ConfUtil.loadJNDI();
		jbServer = jndiProps.getProperty(jbServerProp);
		Container mainPanel = getContentPane();
		mainPanel.setLayout(new VLayout());
		
		mainPanel.add(FormUI.getLabel("JBoss server to connect to", HVProps.defaultLineWidth));
		tfJbServer = FormUI.getTextField(jbServer, 2 * HVProps.defaultLineWidth);
		mainPanel.add(tfJbServer);
		
		cbLoginAnonymous = FormUI.getCheckBox(this, "Login anonymous", 0);
		mainPanel.add(cbLoginAnonymous);
		
		HBox hb = new HBox();
		hb.add(FormUI.getLabel("Username", HVProps.defaultButtonWidth, true, true));
		tfUserName = FormUI.getTextField("", 0);
		hb.add(tfUserName);
		mainPanel.add(hb);

		hb = new HBox();
		hb.add(FormUI.getLabel("Password", HVProps.defaultButtonWidth, true, true));
		pfPasswd = FormUI.getPasswordField("", 0);
		hb.add(pfPasswd);
		mainPanel.add(hb);

		hb = new HBox();
		bConnect = FormUI.getWideButton("Connect");
		bConnect.addActionListener(this);
		hb.add(bConnect);
		HBox hbo = new HBox(HBox.OPPOSITELINEAXIS);
		bExit = FormUI.getButton(this, "Exit");
		bExit.addActionListener(this);
		hbo.add(bExit);
		hb.add(hbo);
		mainPanel.add(hb);
		
		lmopenServers = new DefaultListModel();
		loadServerList();
		lopenServers = new JList(lmopenServers);
		lopenServers.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		lopenServers.setSelectedIndex(0);
		lopenServers.addMouseListener(this);
		mainPanel.add(FormUI.getList(lopenServers, HVProps.defaultLineWidth * 2, 2));
		
		bshowLog = FormUI.getButton(this, "Show log");
		bshowLog.addActionListener(this);
		mainPanel.add(bshowLog);

		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), bConnect);
		getRootPane().getActionMap().put(bConnect, new KeyAction(bConnect, this));
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.ALT_MASK), bExit);
		getRootPane().getActionMap().put(bExit, new KeyAction(bExit, this));
		if (!Settings.appDebug)
			LogUI.makeInvisible();
	}
	
	@SuppressWarnings("unchecked")
	private static void loadServerList() {
		
		Object sl = Settings.get("ServerList");
		if (sl == null) {
			serverList = new ArrayList<JbServerInfo>();
		} else {
			serverList = (ArrayList<JbServerInfo>) sl;
			for (JbServerInfo s : serverList) {
				lmopenServers.addElement(s);
			}
		}
	}

	public JbServerMon getWindow(final String serverURL) {

		if (serverURL == null)
			return null;
		JbServerMon smw = null;
		int i = 0;
		
		while (smw == null && i < childWindows.size()) {
			if (childWindows.get(i) instanceof JbServerMon && serverURL.equals( ((JbServerMon)childWindows.get(i)).getServerURL() ))
				smw = ((JbServerMon)childWindows.get(i));
			i++;
		}
		return smw;
	}

	public void actionPerformed(final ActionEvent ae) {

		if (ae.getSource() == bConnect) {
			bConnect.setEnabled(false);
			final String serverURL = tfJbServer.getText().trim();
			if (serverURL == null || serverURL.length() == 0) {
				UIMessages.showErrorMessage(thisInstance, "No server URL", "A server URL is required.");
				return;
			}
			JbServerMon smw = getWindow(serverURL);
			if (smw != null) {
				Log.log("Opening existing window for connection.");
				smw.setVisible(true);
				smw.toFront();
			} else {
				final MarioContext mc = new MarioContext();
				jndiProps.setProperty(jbServerProp, serverURL);
				if (!cbLoginAnonymous.isSelected()) {
					mc.userName = tfUserName.getText().trim();
					mc.passWord = new String(pfPasswd.getPassword());
				}
				// Do not login, this will hang the application when called a second time.
				mc.connect(null, !cbLoginAnonymous.isSelected(), jndiProps);
				if (mc.ic != null && !JmsUtil.JBossRunning(JmsUtil.getMBeanServer(mc.ic))) {
					mc.close();
				}
				if (mc.ic == null) {
					UIMessages.showErrorMessage(thisInstance, "Connect error", "Could not find running server.");
				}
				if (mc.ic != null) {
					boolean newServerURL = false;
					JbServerInfo serverInfo = getServerInfo(serverURL);
					if (serverInfo == null) {
						serverInfo = new JbServerInfo();
						serverInfo.setServerURL(serverURL);
						serverList.add(serverInfo);
						newServerURL = true;
					}
					serverInfo.setUserName(tfUserName.getText().trim());
					serverInfo.setLoginAnonymous(cbLoginAnonymous.isSelected());
					MBeanServerConnection msc = JmsUtil.getMBeanServer(mc.ic);
					boolean ok = (msc != null && JmsUtil.hasQueueStats(msc));
					if (!JmsUtil.hasQueueStats(msc)) {
						/*
						if (UIMessages.askYesNo(thisInstance, "Stats are off",
								"The message statistics service on the JBoss server is turned off.\n" +
								"This should be enabled via the JBoss console (System> JmxBeans> jboss.messaging> \n" +
								"service=ServerPeer, invoke operation enableMessageCounters().\n" +
								"Try turning it on programmatically?")) {
						 */
						ok = JmsUtil.enableQueueStats(msc, true);
						//}
						if (!ok) {
							UIMessages.showErrorMessage(this, "Queue stats", "Could not enable the queue statistics.");
						}
					}
					if (msc != null) {
						String wTitle = serverURL;
						if (wTitle.startsWith("jnp://"))
							wTitle = wTitle.substring("jnp://".length(), wTitle.length());
						final String fwTitle = wTitle;
						//javax.swing.SwingUtilities.invokeLater(new Runnable() {
						//	public void run() {
								//new JbServerMon(fwTitle);
								// Prevent closing the log from closing the application.
								new JbServerMon(thisInstance, mc, serverURL, fwTitle);
							//}
						//});
						if (newServerURL)
							lmopenServers.addElement(serverInfo);
						//makeInvisible();
					}
				}
			}
			bConnect.setEnabled(true);
		} else if (ae.getSource() == bExit) {
			LogUI.close();
			//exitApp();
		} else if (ae.getSource() == bshowLog) {
			LogUI.makeVisible();
		} else if (ae.getSource() == cbLoginAnonymous) {
			if (cbLoginAnonymous.isSelected()) {
				tfUserName.setEditable(false);
				pfPasswd.setEditable(false);
			} else {
				tfUserName.setEditable(true);
				pfPasswd.setEditable(true);
			}
		}
	}
	
	private JbServerInfo getServerInfo(final String serverURL) {
		
		JbServerInfo jsi = null;
		for (JbServerInfo serverInfo : serverList) {
			if (serverInfo.getServerURL().equals(serverURL)) {
				jsi = serverInfo;
			}
		}
		return jsi;
	}
	
	/**
	 * This method closes the application after storing all appropriate
	 * information if this is the last window that was closed.
	 *
	 */
	public void dispose() {

		// Close all child windows and stores window size.
		super.dispose();
		Settings.put("ServerList", serverList);
		Startup.close(null);
	}
	
	public static void showWindow() {
		
		thisInstance.display(null);
		if (serverList.size() > 0) {
			JbServerInfo serverInfo = serverList.get(0);
			thisInstance.tfJbServer.setText(serverInfo.getServerURL());
			thisInstance.tfUserName.setText(serverInfo.getUserName());
			thisInstance.cbLoginAnonymous.setSelected(serverInfo.isLoginAnonymous());
		}
	}

	public void mouseClicked(final MouseEvent e) {

		if (e.getSource() == lopenServers && 
				SwingUtilities.isLeftMouseButton(e) && lopenServers.getSelectedIndex() > -1) {
			JbServerInfo serverInfo = serverList.get(lopenServers.getSelectedIndex());
			tfJbServer.setText(serverInfo.getServerURL());
			tfUserName.setText(serverInfo.getUserName());
			cbLoginAnonymous.setSelected(serverInfo.isLoginAnonymous());
			if (e.getClickCount() % 2 == 0) { //even clicks, 2,4,6,etc
				actionPerformed(new ActionEvent(bConnect, 0, "mouse-click"));
			}
		}
	}

	public void mouseEntered(final MouseEvent e) {}
	public void mouseExited(final MouseEvent e) {}
	public void mousePressed(final MouseEvent e) {}
	public void mouseReleased(final MouseEvent e) {}
}
