package nl.intercommit.mario.tools.jbossmon;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.swing.SortOrder;
import javax.swing.RowSorter.SortKey;
import javax.swing.event.RowSorterEvent;
import javax.swing.table.TableRowSorter;

import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.MTable;
import net.java.dev.fwutil.UIMessages;
import net.java.dev.hvlayout.HVProps;
import nl.intercommit.mario.tools.JmsUtil;
import nl.intercommit.mario.tools.MarioContext;

public class MsgsTable extends MTable {
	
	private static final long serialVersionUID = 4041311215845946673L;
	private MsgsTableModel tmodel;
	private int selectedModelRow = -1;

	public MsgsTable(final MarioContext mc, final QBrowser smon, final String queueName) {

		super("MsgsTable:" + smon.getServer());
		tmodel = new MsgsTableModel(mc, smon, queueName);
		setSelectionMode(javax.swing.ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		setRowSorter(new TableRowSorter<MsgsTableModel>(tmodel));
		getRowSorter().addRowSorterListener(this);
		setModel(tmodel);
		applyUIProps();
	}
	
    public void sorterChanged(RowSorterEvent e) {
    	
    	if (e.getType() == RowSorterEvent.Type.SORT_ORDER_CHANGED) {
    		selectedModelRow = getSelectedModelRow();
    	}
    	if (e.getType() == RowSorterEvent.Type.SORTED) {
    		if (selectedModelRow > -1) {
    			int viewRow = getRowSorter().convertRowIndexToView(selectedModelRow); 
    			//setRowSelectionInterval(viewRow, viewRow);
    			getSelectionModel().setSelectionInterval(viewRow, viewRow);
    		}
    	}
    }
	
	protected void setDefaultUIProps() {
		// JMS ID
		getColumnModel().getColumn(0).setPreferredWidth((int) (HVProps.defaultButtonWidth * 1.5));
		// JMS Description
		getColumnModel().getColumn(1).setPreferredWidth((int) (HVProps.defaultButtonWidth * 2.5));
		// JMS timestamp
		getColumnModel().getColumn(2).setPreferredWidth((int) (HVProps.defaultButtonWidth * 1.5));
		for (int i = 3; i < getColumnModel().getColumnCount(); i++) {
			getColumnModel().getColumn(i).setPreferredWidth(HVProps.defaultButtonWidth);
		}
		ArrayList<SortKey> skl = new ArrayList<SortKey>();
		skl.add(new SortKey(2, SortOrder.DESCENDING));
		getRowSorter().setSortKeys(skl);
	}
	
	/** Returns the row selected in the table model. */
	public int getSelectedModelRow() {
		
		int row = getSelectedRow(); // The row in the view
		if (row > -1) {
			row = getRowSorter().convertRowIndexToModel(row); // The row in tmodel.
		}
		return row;
	}

	public void dispose() {
		tmodel.dispose();
		super.dispose();
	}
	
	public Message getSelectedMessage() {
		return tmodel.getSelectedMessage(getSelectedModelRow());
	}
	
	public void refresh() {
		tmodel.refresh();
	}
	
	public int[] getSelectedModelRows() {
		
		int[] viewRows = getSelectedRows();
		if (viewRows == null || viewRows.length == 0) {
			return null;
		}
		int[] modelRows = new int[viewRows.length];
		for (int i = 0; i < viewRows.length; i++) {
			modelRows[i] = getRowSorter().convertRowIndexToModel(viewRows[i]);
		}
		return modelRows;
	}
	
	public void saveSelectedMsgs() {
		int[] rows = getSelectedModelRows();
		if (rows == null) {
			UIMessages.showUserErrorMessage(this, "No selection", "One or more messages must be selected to delete.");
		} else {
			boolean error = false;
			int i = 0;
			//MarioContext mc = tmodel.mc;
			String qname = tmodel.qName;
			File out = new File(System.getProperty("java.io.tmpdir")+"\\"+ qname + ".queue");		
			try {
				BufferedWriter buf = new BufferedWriter(new FileWriter(out));
				while (!error && i < rows.length) {
					Message msg = tmodel.getSelectedMessage(rows[i]);
					buf.write(JmsUtil.getMessageContentsAsString(msg));
					buf.flush();
					Log.log("Saved message " + msg.getJMSMessageID());
					buf.write("--------------------------------------------------------------"+"\r\n");
					buf.flush();
					i++;
				}
				buf.close();
			} catch (Exception e) {
				Log.logError(e);
			} finally {
				// Closes all other resources as well.
				//JmsUtil.closeConnection(con);
			}
			UIMessages.showMessage(this, "Message saved", "Message(s) saved to file "+out.getAbsolutePath());
			//refresh();
		}
	}

	public void deleteSelectedMsgs() {
		int[] rows = getSelectedModelRows();
		if (rows == null) {
			UIMessages.showUserErrorMessage(this, "No selection", "One or more messages must be selected to delete.");
		} else {
			boolean error = false;
			int i = 0;
			MarioContext mc = tmodel.mc;
			String qname = tmodel.qName;
			Connection con = null;
			try {
				Queue queue = (Queue)mc.ic.lookup("queue/" + qname);
				ConnectionFactory cf = JmsUtil.getConnectionFactory(mc.ic);
				if (mc.userName != null) {
					con = cf.createConnection(mc.userName, mc.passWord);
				} else {
					con = cf.createConnection();
				}
				Session session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
				con.start();
				while (!error && i < rows.length) {
					Message msg = tmodel.getSelectedMessage(rows[i]);
					String selector = "JMSMessageID = '" + msg.getJMSMessageID() + "'";
					MessageConsumer consumer = session.createConsumer(queue, selector);
					consumer.receive(1000);
					consumer.close();
					Log.log("Deleted message " + msg.getJMSMessageID());
					i++;
				}
			} catch (Exception e) {
				Log.logError(e);
			} finally {
				// Closes all other resources as well.
				JmsUtil.closeConnection(con);
			}
			refresh();
		}
		UIMessages.showMessage(this, "Messages removed", "The selected messages where removed from the queue");
	}
}
