package nl.intercommit.mario.tools.jbossmon;

import java.util.ArrayList;

import org.jboss.jms.server.messagecounter.MessageStatistics;

public class QueueProps {

	public static final String NumberOfMsgs = "MessageCount";
	public static final String NumberOfConsumers = "ConsumerCount";
	public static final int avgPeriod = 10;

	public static final String[] queueAtts = new String[] {NumberOfMsgs,
		NumberOfConsumers
	};

	public int queueDepth;
	public int queueDepthPeak;
	public int msgConsumers;
	public int msgTotal;
	public int avgPerSecLast;
	public int avgPerSecPeriod;
	public int avgPerSecPeak;

	private long lastUpdate = 0;
	private ArrayList<StatProp> statProps;

	public QueueProps() {
		super();
		setInitValues();
	}

	public void setInitValues() {

		statProps = new ArrayList<StatProp>();
		queueDepth = -1;
		queueDepthPeak = -1;
		msgConsumers = -1;
		msgTotal = -1;
		avgPerSecPeriod = -1;
		avgPerSecPeak = -1;
		avgPerSecLast = -1;
	}

	/** Returns null on succes, else an error message.
	 * @param ms
	 */
	public String addStats(final MessageStatistics ms) {

		String error = null;
		if (ms == null || ms.getTimeLastUpdate() == lastUpdate) {
			if (ms == null) {
				error = "Got no stats to process.";
				//Log.logError(error);
				return error;
			}
			error = "Queue stats have not been updated by server (" + ms.getName() + ").";
			//Log.logWarning(error);
			return error;
		}
		lastUpdate = ms.getTimeLastUpdate();
		msgTotal = ms.getCount();
		StatProp lastStats = new StatProp(lastUpdate, ms.getCount(), ms.getDepth());
		// Add new statistics to the end of the list.
		if (statProps.size() > 1) {
			lastStats.queueDelta = lastStats.queueDepth - statProps.get(statProps.size()-1).queueDepth;
		}
		statProps.add(lastStats);
		if (statProps.size() > avgPeriod) {
			statProps.remove(0);
		}
		if (statProps.size() > 1) {
			long avgtime = lastStats.timeStamp - statProps.get(0).timeStamp;
			int avgmsgs = lastStats.msgCount - statProps.get(0).msgCount
			- getQueueDeltaTotal();
			avgPerSecPeriod = (int) (avgmsgs / (avgtime / 1000));
			avgtime = statProps.get(statProps.size()-1).timeStamp - statProps.get(statProps.size()-2).timeStamp;
			avgmsgs = statProps.get(statProps.size()-1).msgCount - statProps.get(statProps.size()-2).msgCount
			- lastStats.queueDelta;
			avgPerSecLast = (int) (avgmsgs / (avgtime / 1000));
			if (avgPerSecPeriod > avgPerSecPeak && statProps.size() > avgPeriod - 1)
				avgPerSecPeak = avgPerSecPeriod;
		}
		return null;
	}

	private int getQueueDeltaTotal() {

		int delta = 0;
		for (int i = 0; i < statProps.size(); i++) {
			delta += statProps.get(i).queueDelta;
		}
		return delta;
	}

	public void clearStats() {

		statProps.clear();
		queueDepthPeak = -1;
		avgPerSecLast = -1;
		avgPerSecPeriod = -1;
		avgPerSecPeak = -1;
	}

	class StatProp {

		public long timeStamp;
		public int msgCount;
		public int queueDepth;
		public int queueDelta = 0;
		public StatProp(final long timeStamp, final int msgCount, final int queueDepth) {
			this.timeStamp = timeStamp;
			this.msgCount = msgCount;
			this.queueDepth = queueDepth;
		}
	}
}
